#include "../inbox.h"

// Submodules
#include "ap/object.h"

// Model
#include "model/account.h"
#include "model/status.h"

// Standard Library
#include <string.h>

bool route_add( struct ap_object* act )
{
	bool result = false;
	struct status* s = NULL;
	struct account* a = NULL;
	struct ap_object* apa = NULL;

	// Make sure this is to pin a status
	a = account_from_uri_or_fetch( act->actor );
	if( !a ) {
		printf( "Don't have account...\n" );
		goto failed;
	} // Don't have ActivityPub account data, try again later
	apa = account_get_activity_pub_data(a);
	if( !apa ) {
		printf( "Failed to get activity pub data for account %s\n", act->actor );
		goto failed;
	}
	if( 0 != strcmp( apa->featured, act->target ) ) {
		printf( "Not adding to featured...\n" );
		printf( "apa->featured = %s\n", apa->featured );
		printf( "act->target = %s\n", act->target );
		goto failed;
	}

	// Get the status
	if( act->object.tag == apaot_ref ) {
		s = status_from_uri_or_fetch( act->object.ref );
		if( !s ) {
			printf( "Unable to get status for %s\n", act->object.ref );
			goto failed;
		}
	} else if( act->object.ptr->type == ap_Note ) {
		printf( "TODO: handle embedded status\n" );
		goto failed;
	} else {
		printf( "Unsupported embedded object type\n" );
		goto discard;
	}

	s->pinned = true;
	status_save(s);

	goto discard;
cleanup:
	status_free(s);
	ap_object_free(apa);
	account_free(a);
	return result;
failed:
	result = false;
	goto cleanup;
discard:
	result = true;
	goto cleanup;
}
