#include "model/account.h"

// Submodules
#include "util/format.h"
#include "ffdb/trie.h"
#include "collections/array.h"
#include "ap/object.h"

// Model
#include "model/server.h"
#include "model/status.h"
#include "model/activity.h"
#include "model/timeline.h"

// Standard Library
#include <stdlib.h>
#include <string.h>

/*
struct ap_object* account_activity_pub( struct account* a )
{
	// Pass thru remote activity pub actor data if present
	char filename[512];
	//struct ap_object* obj = ap_object_from_file( format( filename, 512, "data/accounts/%d/ap.json", a->id ) );
	if( obj ) { return obj; }

	return account_ap_actor(a);
}
*/

struct ap_object* account_ap_actor( struct account* a )
{
	struct ap_object* act = activity_new_local_activity();
	act->ap_context.language = clang_undefined;
	act->type = ap_Person;
	act->id = aformat( "https://%s/owner/actor", g_server->domain );
	act->name = strdup(a->display_name);
	act->summary = strdup(a->note);
	act->endpoints.shared_inbox = aformat("https://%s/inbox", g_server->domain);
	act->inbox = aformat("https://%s/inbox", g_server->domain);
	act->outbox = aformat("https://%s/outbox", g_server->domain );
	act->preferred_username = strdup(a->handle);
	act->featured = aformat("https://%s/owner/collections/featured", g_server->domain );
	act->followers = aformat("https://%s/owner/followers", g_server->domain);
	act->following = aformat("https://%s/owner/following", g_server->domain);
	act->also_known_as.items = malloc(1);
	act->discoverable.visible = true;
	act->discoverable.value = false;
	act->manually_approves_followers.visible = true;
	act->manually_approves_followers.value = false;

	act->tags.items = malloc(1);

	struct ap_object* avatar = ap_object_new();
	avatar->type = ap_Image;
	ap_object_array_append_ref( &avatar->url, aformat( "https://%s/owner/avatar.blob", g_server->domain ) );
	avatar->media_type = strdup( "image/png" );
	act->icon = avatar;

	struct ap_object* banner = ap_object_new();
	banner->type = ap_Image;
	banner->media_type = strdup( "image/png" );
	ap_object_array_append_ref( &banner->url, aformat( "https://%s/owner/banner.blob", g_server->domain ) );

	act->image = banner;
	ap_object_array_append_ref( &act->url, aformat( "https://%s/@%s@%s", g_server->domain, a->handle, g_server->domain ) );
	act->capabilities.show = true;

	struct ap_public_key* key;
	key = malloc(sizeof(*key));
	memset(key,0,sizeof(*key));

	key->id = aformat( "https://%s/owner/actor#mainKey", g_server->domain );
	key->owner = strdup(act->id);
	FILE* f = fopen( "data/owner/public.pem", "r" );
	fseek( f, 0, SEEK_END );
	int size = ftell(f);
	fseek( f, 0, SEEK_SET );
	char* pem = malloc( size + 2 );
	fread( pem, 1, size, f );
	pem[size+0] = '\n';
	pem[size+1] = '\0';
	key->public_key = pem;
	fclose(f);

	act->public_key = key;

	for( int i = 0; i < a->fields.count; ++i ) {
		struct ap_object* at;
		at = malloc(sizeof(*at));
		memset(at,0,sizeof(*at));

		at->name = strdup(a->fields.items[i].key);
		at->value = strdup(a->fields.items[i].value);
		at->type = schema_PropertyValue;

		array_append( &act->attachments, sizeof(at), &at );
	}

	return act;
}

struct ap_object* account_ap_outbox( struct account* a )
{
	struct ap_object* outbox = activity_new_local_activity();
	outbox->type = ap_OrderedCollection;
	outbox->published = time(NULL);
	outbox->id = aformat( "https://%s/outbox", g_server->domain );
	outbox->first.tag = apaot_ref;
	outbox->first.ref = aformat( "https://%s/outbox/page-0", g_server->domain );

	char path[512];
	account_get_path( a->id, path, 512 );

	char buffer[512];
	outbox->total_items = ffdb_trie_count( format( buffer, 512, "%s/timeline", path ) );

	return outbox;
}
struct ap_object* account_ap_outbox_page( struct account* a, int page )
{
	enum { items_per_page = 10 };
	char path[512];
	account_get_path( a->id, path, 512 );

	char buffer[512];
	int total_items = ffdb_trie_count( format( buffer, 512, "%s/timeline", path ) );
	int page_count = ( total_items + items_per_page - 1 ) / items_per_page;

	if( page >= page_count ) { return NULL; }

	struct ap_object* outbox = activity_new_local_activity();

	struct {
		char** items;
		int count;
	} values;
	memset( &values, 0, sizeof(values) );
	ffdb_trie_list( buffer, page * items_per_page, items_per_page, NULL, &values );
	for( int i = 0; i < values.count; ++i ) {
		int id;
		int activity_id = 0;
		if( 1 == sscanf( values.items[i], "act-%d", &activity_id ) ) {
			// Direct activity
		} else if( 1 == sscanf( values.items[i], "%d", &id ) ) {
			struct status* s = status_from_id(id);
			if( !s ) { goto include_tombstone; }

			activity_id = s->activity_id;
			if( s->activity_id == 0 ) {
				goto include_tombstone;
			}
			status_free(s);
		}

		if( activity_id ) {
			struct ap_object* act = activity_from_local_id( activity_id );
			if( !act ) { goto include_tombstone; }

			struct ap_object_ptr_or_ref r;
			r.tag = apaot_activity;
			r.ptr = act;

			array_append( &outbox->collection_items, sizeof(r), &r );
		} else {
			include_tombstone:
			// include Tombstone
			struct ap_object* o;
			o = malloc(sizeof(*o));
			memset(o,0,sizeof(*o));

			if( activity_id != 0 ) {
				o->id = aformat( "https://%s/activity/%d", g_server->domain, activity_id );
			} else {
				o->id = aformat( "https://%s/activity/tombstone", g_server->domain );
			}
			o->type = ap_Tombstone;

			struct ap_object_ptr_or_ref r;
			r.tag = apaot_activity;
			r.ptr = o;

			array_append( &outbox->collection_items, sizeof(r), &r );
		}
		free( values.items[i] );
	}
	free( values.items );

	outbox->type = ap_OrderedCollectionPage;
	outbox->published = time(NULL);
	outbox->part_of = aformat( "https://%s/outbox", g_server->domain );
	outbox->id = aformat( "https://%s/outbox/page-%d", g_server->domain, page );
	if( page > 0 ) {
		outbox->prev = aformat( "https://%s/outbox/page-%d", g_server->domain, page - 1 );
	}
	if( page < page_count - 1 ) {
		outbox->next.tag = apaot_ref;
		outbox->next.ref = aformat( "https://%s/outbox/page-%d", g_server->domain, page + 1 );
	}

	return outbox;
}
static struct ap_object* account_list_page( int page, char* part_of, const char* page_format, const char* trie_filename )
{
	enum { items_per_page = 50 };
	int total_items = ffdb_trie_count( trie_filename );
	int page_count = ( total_items + items_per_page - 1 ) / items_per_page;

	if( page >= page_count ) { return NULL; }

	struct ap_object* o = activity_new_local_activity();
	o->type = ap_OrderedCollectionPage;
	o->published = time(NULL);
	o->part_of = part_of;
	o->id = aformat( page_format, page );
	if( page > 0 ) {
		o->prev = aformat( page_format, page - 1 );
	}
	if( page < page_count - 1 ) {
		o->next.tag = apaot_ref;
		o->next.ref = aformat( page_format, page + 1 );
	}

	struct {
		char** items;
		int count;
	} values;
	memset( &values, 0, sizeof(values) );
	ffdb_trie_list( trie_filename, page * items_per_page, items_per_page, &values, NULL );
	for( int i = 0; i < values.count; ++i ) {
		//printf( "values[%d] = %s\n", i, values.items[i] );
		int id;
		if( 1 != sscanf( values.items[i], "%d", &id ) ) { continue; }
		free( values.items[i] );

		struct account* follower = account_from_id(id);
		if( !follower ) { continue; }

		struct ap_object_ptr_or_ref r;
		r.tag = apaot_ref;
		r.ref = strdup( follower->account_url );
		array_append( &o->ordered_items, sizeof(r), &r );
		account_free(follower);
	}
	free( values.items );

	return o;
}

struct ap_object* account_ap_followers( struct account* a )
{
	if( a->id != 0 ) { return NULL; }

	struct ap_object* o = activity_new_local_activity();

	o->type = ap_OrderedCollection;
	o->published = time(NULL);
	o->id = aformat( "https://%s/owner/followers", g_server->domain );
	o->first.tag = apaot_object;
	o->first.ptr = account_ap_followers_page( a, 0 );

	char path[512];
	account_get_path( a->id, path, 512 );

	char buffer[512];
	o->total_items = ffdb_trie_count( format( buffer, 512, "%sd/followers", path ) );

	return o;
}

struct ap_object* account_ap_followers_page( struct account* a, int page )
{
	if( a->id != 0 ) { return NULL; }

	char* part_of = aformat( "https://%s/owner/followers", g_server->domain );
	char page_format[512];
	snprintf( page_format, 512, "https://%s/owner/followers/page-%%d", g_server->domain );

	char path[512];
	account_get_path( a->id, path, 512 );

	char trie_filename[512];
	snprintf( trie_filename, 512, "%s/followers", path );

	return account_list_page( page, part_of, page_format, trie_filename );
}

struct ap_object* account_ap_following( struct account* a )
{
	if( a->id != 0 ) { return NULL; }

	struct ap_object* o = activity_new_local_activity();

	o->type = ap_OrderedCollection;
	o->published = time(NULL);
	o->id = aformat( "https://%s/owner/following", g_server->domain );
	o->first.tag = apaot_object;
	o->first.ptr = account_ap_following_page( a, 0 );

	char path[512];
	account_get_path( a->id, path, 512 );

	char buffer[512];
	o->total_items = ffdb_trie_count( format( buffer, 512, "%s/following", path ) );

	return o;
}

struct ap_object* account_ap_following_page( struct account* a, int page )
{
	if( a->id != 0 ) { return NULL; }

	char* part_of = aformat( "https://%s/owner/following", g_server->domain );
	char page_format[512];
	snprintf( page_format, 512, "https://%s/owner/following/page-%%d", g_server->domain );

	char path[512];
	account_get_path( a->id, path, 512 );

	char trie_filename[512];
	snprintf( trie_filename, 512, "%s/following", path );

	return account_list_page( page, part_of, page_format, trie_filename );
}

struct ap_object* account_ap_featured( struct account* a )
{
	struct ap_object* o = activity_new_local_activity();
	o->type = ap_OrderedCollection;
	o->ap_context.language = clang_undefined;
	//o->published = time(NULL);
	char path[512];
	account_get_path( a->id, path, 512 );

	char buffer[512];
	o->id = aformat( "https://%s/owner/collections/featured", g_server->domain );
	o->total_items = ffdb_trie_count( format( buffer,512, "%s/timeline/pinned", path ) );
	o->ordered_items.items = malloc(1);

	if( o->total_items > 0 ) {
		o->first.tag = apaot_object;
		o->first.ptr = account_ap_featured_page( a, 0 );
	}
	return o;
}
struct ap_object* account_ap_featured_page( struct account* a, int page )
{
	char path[512];
	account_get_path( a->id, path, 512 );

	char buffer[512];
	snprintf( buffer,512, "%s/timeline/pinned", path );
	struct timeline* tl = timeline_from_path( buffer );
	if( !tl ) { return NULL; }

	int total_items = ffdb_trie_count( format( buffer,512, "%s/timeline/pinned", path ) );

	enum { items_per_page = 20 };
	const int total_pages = ( total_items + items_per_page - 1 ) / items_per_page;

	struct status* statuses[items_per_page];
	int count = timeline_load_statuses( tl, page * items_per_page, items_per_page, statuses );

	struct ap_object* o = ap_object_new();
	o->type = ap_OrderedCollectionPage;
	o->part_of = aformat( "https://%s/owner/collections/featured", g_server->domain );
	o->id = aformat( "%s/page-%d", o->part_of, page);
	if( page+1 < total_pages ) {
		o->next.tag = apaot_ref;
		o->next.ref = aformat( "%s/page-%d", o->part_of, page+1 );
	}
	if( page > 0 ) {
		o->prev = aformat( "%s/page-%d", o->part_of, page-11 );
	}

	o->ordered_items.items = malloc(1);

	for( int i = 0; i < count; ++i ) {
		struct ap_object_ptr_or_ref r;
		r.tag = apaot_ref;
		r.ref = strdup(statuses[i]->url);
		array_append( &o->ordered_items, sizeof(r), &r );
		status_free(statuses[i]);
	}

	timeline_free(tl);
	return o;
}

